"""make calibration fetch conditional

Revision ID: f1d5915e3450
Revises: e47383255e83
Create Date: 2021-10-28 15:29:28.112760

"""
from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = "f1d5915e3450"
down_revision = "e47383255e83"
branch_labels = None
depends_on = None


script = """#!/bin/sh
set -o errexit

export HOME=$TMPDIR

{{^resubmit}}
./framework.sh -d .
chmod 770 .
cd rawdata/
../productfetcher --product-locator $1
cd ../
{{/resubmit}}

{{#resubmit}}
./framework.sh -wp .
chmod 770 .
{{/resubmit}}
"""

condor = """executable = std_calibration_fetch.sh
arguments = {{product_locator}}

output = calibration_fetch.out
error = calibration_fetch.err
log = condor.log

SBIN_PATH = /lustre/aoc/cluster/pipeline/$ENV(CAPO_PROFILE)/workspaces/sbin
should_transfer_files = yes
transfer_input_files = $(SBIN_PATH)/pycapo, $(SBIN_PATH)/framework.sh, $(SBIN_PATH)/productfetcher{{#resubmit}}, {{rawdata_dir}}{{/resubmit}}
transfer_output_files = working, rawdata, products

getenv = True
environment = "CAPO_PATH=/home/casa/capo"

requirements = HasLustre == True

queue
"""

old_script = """#!/bin/sh
set -o errexit

export HOME=$TMPDIR

./framework.sh -d .
chmod 770 .
cd rawdata/
../productfetcher --product-locator $1
cd ../

"""

old_condor = """executable = std_calibration_fetch.sh
arguments = {{product_locator}}

output = calibration_fetch.out
error = calibration_fetch.err
log = condor.log

SBIN_PATH = /lustre/aoc/cluster/pipeline/$ENV(CAPO_PROFILE)/workspaces/sbin
should_transfer_files = yes
transfer_input_files = $(SBIN_PATH)/pycapo, $(SBIN_PATH)/framework.sh, $(SBIN_PATH)/productfetcher
transfer_output_files = working, rawdata, products

getenv = True
environment = "CAPO_PATH=/home/casa/capo"

requirements = HasLustre == True

queue

"""


def upgrade():
    op.execute(
        f"""
        UPDATE workflow_templates
        SET content=E'{script}' WHERE filename='std_calibration_fetch.sh'
        """
    )
    op.execute(
        f"""
        UPDATE workflow_templates
        SET content=E'{condor}' WHERE filename='std_calibration_fetch.condor'
        """
    )


def downgrade():
    op.execute(
        f"""
        UPDATE workflow_templates
        SET content=E'{old_script}' WHERE filename='std_calibration_fetch.sh'
        """
    )
    op.execute(
        f"""
            UPDATE workflow_templates
            SET content=E'{old_condor}' WHERE filename='std_calibration_fetch.condor'
            """
    )
