#
# Copyright (C) 2021 Associated Universities, Inc. Washington DC, USA.
#
# This file is part of NRAO Workspaces.
#
# Workspaces is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Workspaces is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Workspaces.  If not, see <https://www.gnu.org/licenses/>.
""" Notification endpoint routes """

from pyramid.config import Configurator


def includeme(config: Configurator):
    """
    Function that gets included in server.py::main(); calls all route adding functions

    :param config: Pyramid server config object
    """
    default_routes(config)
    notification_routes(config)
    notification_sending_routes(config)


def default_routes(config: Configurator):
    """
    Uncategorized server routes

    :param config: Pyramid server config object
    """
    config.add_route("home", "/")


def notification_routes(config: Configurator):
    """
    Server routes for creating, updating, and removing Notification templates

    :param config: Pyramid server config object
    """
    config.add_route(name="notifications", pattern="/notify")
    config.add_route(name="notification", pattern="/notify/{template_name}")
    config.add_route(
        name="update_notification_template",
        pattern="/update_notification_template/{template_name}/content/{text}",
        request_method="POST",
    )


def notification_sending_routes(config: Configurator):
    """
    Server routes for sending Notifications

    :param config: Pyramid server config object
    """
    config.add_route(name="send_notification", pattern="/notify/{template_name}/send", request_method="POST")
